@@ -11,8 +11,8 @@
 
 struct demo
 {
-    char   name[PATHMAX];     /* demo basename    */
-    char   filename[MAXSTR];  /* demo path        */
+    char   name[PATHMAX];               /* Demo basename                     */
+    char   filename[MAXSTR];            /* Demo path                         */
 
     char   player[MAXSTR];
     time_t date;
@@ -22,15 +22,15 @@ struct demo
     int    status;
     int    mode;
 
-    char   shot[PATHMAX];   /* image filename */
-    char   file[PATHMAX];   /* level filename */
+    char   shot[PATHMAX];               /* Image filename                    */
+    char   file[PATHMAX];               /* Level filename                    */
+
+    int    time;                        /* Time limit                        */
+    int    goal;                        /* Coin limit                        */
+    int    score;                       /* Total coins                       */
+    int    balls;                       /* Number of balls                   */
+    int    times;                       /* Total time                        */
 
-    int    time;            /* time limit        */
-    int    goal;            /* coin limit        */
-    int    goal_e;          /* goal enabled flag */
-    int    score;           /* total coins       */
-    int    balls;           /* number of balls   */
-    int    times;           /* total time        */
 };
 
 /*---------------------------------------------------------------------------*/
@@ -38,7 +38,7 @@ struct demo
 struct demo *demo_load(const char *);
 void         demo_free(struct demo *);
 
-int  demo_exists(const char *);
+int demo_exists(const char *);
 
 const char *demo_format_name(const char *fmt,
                              const char *set,
@@ -46,8 +46,7 @@ const char *demo_format_name(const char *fmt,
 
 /*---------------------------------------------------------------------------*/
 
-int  demo_play_init(const char *, const struct level *,
-                    int, int, int, int, int, int, int);
+int  demo_play_init(const char *, const struct level *, int, int, int, int);
 void demo_play_step(void);
 void demo_play_stat(int, int, int);
 void demo_play_stop(void);
@@ -62,12 +61,15 @@ void demo_rename_player(const char *name, const char *player);
 int  demo_replay_init(const char *, int *, int *, int *, int *, int *);
 int  demo_replay_step(float);
 void demo_replay_stop(int);
+float demo_replay_blend(void);
 
-const struct demo *curr_demo_replay(void);
+const char *curr_demo(void);
+
+void demo_speed_set(int);
 
 /*---------------------------------------------------------------------------*/
 
-fs_file demo_file(void);
+extern fs_file demo_fp;
 
 /*---------------------------------------------------------------------------*/
 
