@@ -3,14 +3,18 @@
 
 #include "base_config.h"
 #include "score.h"
-#include "progress.h"
 
 /*---------------------------------------------------------------------------*/
 
+enum
+{
+    SCORE_TIME = 0,
+    SCORE_GOAL,
+    SCORE_COIN
+};
+
 struct level
 {
-    /* TODO: turn into an internal structure. */
-
     char file[PATHMAX];
     char shot[PATHMAX];
     char song[PATHMAX];
@@ -23,18 +27,10 @@ struct level
     int time; /* Time limit   */
     int goal; /* Coins needed */
 
-    struct
-    {
-        struct score best_times;
-        struct score fast_unlock;
-        struct score most_coins;
-    }
-    score;
+    struct score scores[3];
 
     /* Set information. */
 
-    struct set *set;
-
     int  number;
 
     /* String representation of the number (eg. "IV") */
@@ -43,33 +39,38 @@ struct level
     int is_locked;
     int is_bonus;
     int is_completed;
+
+    struct level *next;
 };
 
 int  level_load(const char *, struct level *);
 
 /*---------------------------------------------------------------------------*/
 
-int  level_exists(int);
+int level_exists(int);
 
-void level_open  (int);
-int  level_opened(int);
+void level_open(struct level *);
+int level_opened(const struct level *);
 
-void level_complete (int);
-int  level_completed(int);
+void level_complete(struct level *);
+int level_completed(const struct level *);
 
-int  level_time(int);
-int  level_goal(int);
-int  level_bonus(int);
+int level_time(const struct level *);
+int level_goal(const struct level *);
+int level_bonus(const struct level *);
 
-const char *level_shot(int);
-const char *level_file(int);
-const char *level_name(int);
-const char *level_msg (int);
+const char *level_shot(const struct level *);
+const char *level_file(const struct level *);
+const char *level_song(const struct level *);
+const char *level_name(const struct level *);
+const char *level_msg(const struct level *);
+
+const struct score *level_score(struct level *, int);
 
 /*---------------------------------------------------------------------------*/
 
-int  level_score_update (int, int, int, int *, int *, int *);
-void level_rename_player(int, int, int, int, const char *);
+int  level_score_update (struct level *, int, int, int *, int *, int *);
+void level_rename_player(struct level *, int, int, int, const char *);
 
 /*---------------------------------------------------------------------------*/
 
