@@ -1,11 +1,13 @@
 #ifndef PROGRESS_H
 #define PROGRESS_H
 
+#include "level.h"
+
 /*---------------------------------------------------------------------------*/
 
 void progress_init(int);
 
-int  progress_play(int level);
+int  progress_play(struct level *);
 void progress_step(void);
 void progress_stat(int status);
 void progress_stop(void);
@@ -27,7 +29,8 @@ int  progress_last(void);
 int  progress_lvl_high(void);
 int  progress_set_high(void);
 
-int  curr_level(void);
+struct level *curr_level(void);
+
 int  curr_balls(void);
 int  curr_score(void);
 int  curr_mode (void);
@@ -51,6 +54,7 @@ enum
 
     MODE_CHALLENGE,
     MODE_NORMAL,
+    MODE_STANDALONE,
 
     MODE_MAX
 };
