@@ -21,21 +21,18 @@
 #include "progress.h"
 #include "audio.h"
 #include "config.h"
-#include "st_shared.h"
 
 #include "game_common.h"
 
 #include "st_done.h"
-#include "st_start.h"
+#include "st_level.h"
 #include "st_name.h"
+#include "st_shared.h"
 
 /*---------------------------------------------------------------------------*/
 
 #define DONE_OK   1
 
-/* Bread crumbs. */
-
-static int new_name;
 static int resume;
 
 static int done_action(int i)
@@ -45,23 +42,21 @@ static int done_action(int i)
     switch (i)
     {
     case DONE_OK:
-        return goto_state(&st_start);
+        return goto_state(&st_exit);
 
     case GUI_NAME:
-        new_name = 1;
         return goto_name(&st_done, &st_done, 0);
 
-    case GUI_MOST_COINS:
-    case GUI_BEST_TIMES:
-    case GUI_FAST_UNLOCK:
+    case GUI_SCORE_COIN:
+    case GUI_SCORE_TIME:
+    case GUI_SCORE_GOAL:
         gui_score_set(i);
-        resume = 1;
         return goto_state(&st_done);
     }
     return 1;
 }
 
-static int done_enter(void)
+static int done_gui(void)
 {
     const char *s1 = _("New Set Record");
     const char *s2 = _("Set Complete");
@@ -70,11 +65,6 @@ static int done_enter(void)
 
     int high = progress_set_high();
 
-    if (new_name)
-    {
-        progress_rename(1);
-        new_name = 0;
-    }
 
     if ((id = gui_vstack(0)))
     {
@@ -86,7 +76,7 @@ static int done_enter(void)
             gid = gui_label(id, s2, GUI_MED, GUI_ALL, gui_blu, gui_grn);
 
         gui_space(id);
-        gui_score_board(id, GUI_MOST_COINS | GUI_BEST_TIMES, 1, high);
+        gui_score_board(id, GUI_SCORE_COIN | GUI_SCORE_TIME, 1, high);
         gui_space(id);
 
         gui_start(id, _("Select Level"), GUI_SML, DONE_OK, 0);
@@ -97,16 +87,23 @@ static int done_enter(void)
         gui_layout(id, 0, 0);
     }
 
-    set_score_board(set_coin_score(curr_set()), progress_score_rank(),
-                    set_time_score(curr_set()), progress_times_rank(),
+    set_score_board(set_score(curr_set(), SCORE_COIN), progress_score_rank(),
+                    set_score(curr_set(), SCORE_TIME), progress_times_rank(),
                     NULL, -1);
 
-    /* Reset hack. */
-    resume = 0;
-
     return id;
 }
 
+static int done_enter(struct state *st, struct state *prev)
+{
+    if (prev == &st_name)
+        progress_rename(1);
+
+    resume = (prev == &st_done);
+
+    return done_gui();
+}
+
 static int done_keybd(int c, int d)
 {
     if (d && config_tst_d(CONFIG_KEY_SCORE_NEXT, c))
@@ -139,6 +136,5 @@ struct state st_done = {
     shared_angle,
     shared_click,
     done_keybd,
-    done_buttn,
-    1, 0
+    done_buttn
 };
