@@ -28,22 +28,18 @@
 
 #include "st_play.h"
 #include "st_goal.h"
-#include "st_fall_out.h"
-#include "st_time_out.h"
-#include "st_over.h"
+#include "st_fail.h"
 #include "st_pause.h"
+#include "st_level.h"
 #include "st_shared.h"
 
 /*---------------------------------------------------------------------------*/
 
-static float view_rotate;
-static int   fast_rotate;
-
 static int pause_or_exit(void)
 {
     if (config_tst_d(CONFIG_KEY_PAUSE, SDLK_ESCAPE))
     {
-        return goto_pause();
+        return goto_state(&st_pause);
     }
     else
     {
@@ -52,7 +48,7 @@ static int pause_or_exit(void)
 
         video_clr_grab();
 
-        return goto_state(&st_over);
+        return goto_state(&st_exit);
     }
 }
 
@@ -81,9 +77,6 @@ static void keybd_camera(int c)
     if (config_tst_d(CONFIG_KEY_CAMERA_3, c))
         set_camera(VIEW_MANUAL);
 
-    if (c == SDLK_F4 && config_cheat())
-        set_camera(VIEW_TOPDOWN);
-
     if (config_tst_d(CONFIG_KEY_CAMERA_TOGGLE, c))
         toggle_camera();
 }
@@ -116,7 +109,7 @@ static void buttn_camera(int b)
 
 /*---------------------------------------------------------------------------*/
 
-static int play_ready_enter(void)
+static int play_ready_gui(void)
 {
     int id;
 
@@ -126,17 +119,22 @@ static int play_ready_enter(void)
         gui_pulse(id, 1.2f);
     }
 
+    return id;
+}
+
+static int play_ready_enter(struct state *st, struct state *prev)
+{
     audio_play(AUD_READY, 1.0f);
     video_set_grab(1);
 
     hud_view_pulse(config_get_d(CONFIG_CAMERA));
 
-    return id;
+    return play_ready_gui();
 }
 
 static void play_ready_paint(int id, float t)
 {
-    game_draw(0, t);
+    game_client_draw(0, t);
     hud_view_paint();
     gui_paint(id);
 }
@@ -145,8 +143,7 @@ static void play_ready_timer(int id, float dt)
 {
     float t = time_state();
 
-    game_set_fly(1.0f - 0.5f * t, NULL);
-    game_client_step(NULL);
+    game_client_fly(1.0f - 0.5f * t);
 
     if (dt > 0.0f && t > 1.0f)
         goto_state(&st_play_set);
@@ -175,7 +172,7 @@ static int play_ready_keybd(int c, int d)
         keybd_camera(c);
 
         if (config_tst_d(CONFIG_KEY_PAUSE, c))
-            goto_pause();
+            goto_state(&st_pause);
     }
     return 1;
 }
@@ -196,7 +193,7 @@ static int play_ready_buttn(int b, int d)
 
 /*---------------------------------------------------------------------------*/
 
-static int play_set_enter(void)
+static int play_set_gui(void)
 {
     int id;
 
@@ -206,16 +203,19 @@ static int play_set_enter(void)
         gui_pulse(id, 1.2f);
     }
 
+    return id;
+}
+
+static int play_set_enter(struct state *st, struct state *prev)
+{
     audio_play(AUD_SET, 1.f);
 
-    clear_pause();
-
-    return id;
+    return play_set_gui();
 }
 
 static void play_set_paint(int id, float t)
 {
-    game_draw(0, t);
+    game_client_draw(0, t);
     hud_view_paint();
     gui_paint(id);
 }
@@ -224,8 +224,7 @@ static void play_set_timer(int id, float dt)
 {
     float t = time_state();
 
-    game_set_fly(0.5f - 0.5f * t, NULL);
-    game_client_step(NULL);
+    game_client_fly(0.5f - 0.5f * t);
 
     if (dt > 0.0f && t > 1.0f)
         goto_state(&st_play_loop);
@@ -254,7 +253,7 @@ static int play_set_keybd(int c, int d)
         keybd_camera(c);
 
         if (config_tst_d(CONFIG_KEY_PAUSE, c))
-            goto_pause();
+            goto_state(&st_pause);
     }
     return 1;
 }
@@ -275,53 +274,66 @@ static int play_set_buttn(int b, int d)
 
 /*---------------------------------------------------------------------------*/
 
+struct
+{
+    float R;
+    float L;
+
+    enum
+    {
+        DIR_R = 1,
+        DIR_L
+    } d;
+} view_rotate;
+
+#define VIEWR_SET_R(v) do {                                    \
+    view_rotate.R = (v);                                       \
+    view_rotate.d = (v) ? DIR_R : (view_rotate.L ? DIR_L : 0); \
+} while (0)
+
+#define VIEWR_SET_L(v) do {                                    \
+    view_rotate.L = (v);                                       \
+    view_rotate.d = (v) ? DIR_L : (view_rotate.R ? DIR_R : 0); \
+} while (0)
+
+static int fast_rotate;
 static int show_hud;
 
-static int play_loop_enter(void)
+static int play_loop_gui(void)
 {
-    union cmd cmd;
     int id;
 
-    if (is_paused())
-    {
-        clear_pause();
-        view_rotate = 0;
-        fast_rotate = 0;
-        return 0;
-    }
-
     if ((id = gui_label(0, _("GO!"), GUI_LRG, GUI_ALL, gui_blu, gui_grn)))
     {
         gui_layout(id, 0, 0);
         gui_pulse(id, 1.2f);
     }
 
-    audio_play(AUD_GO, 1.f);
+    return id;
+}
 
-    game_set_fly(0.f, NULL);
-
-    /* End first update. */
-
-    cmd.type = CMD_END_OF_UPDATE;
-    game_proxy_enq(&cmd);
-
-    /* Sync client. */
-
-    game_client_step(demo_file());
-
-    view_rotate = 0;
+static int play_loop_enter(struct state *st, struct state *prev)
+{
+    VIEWR_SET_R(0);
+    VIEWR_SET_L(0);
     fast_rotate = 0;
 
-    show_hud = 1;
+    if (prev == &st_pause)
+        return 0;
 
+    audio_play(AUD_GO, 1.f);
+
+    game_client_fly(0.0f);
+
+    show_hud = 1;
     hud_update(0);
 
-    return id;
+    return play_loop_gui();
 }
 
 static void play_loop_paint(int id, float t)
 {
-    game_draw(0, t);
+    game_client_draw(0, t);
 
     if (show_hud)
         hud_paint();
@@ -338,32 +350,32 @@ static void play_loop_timer(int id, float dt)
 
     gui_timer(id, dt);
     hud_timer(dt);
-    game_set_rot(view_rotate * k);
+    game_set_rot(view_rotate.d == DIR_R ?
+                 view_rotate.R * k :
+                 view_rotate.L * k);
     game_set_cam(config_get_d(CONFIG_CAMERA));
 
     game_step_fade(dt);
 
     game_server_step(dt);
-    game_client_step(demo_file());
+    game_client_sync(demo_fp);
+    game_client_blend(game_server_blend());
 
     switch (curr_status())
     {
     case GAME_GOAL:
         progress_stat(GAME_GOAL);
-        gui_stuck();
         goto_state(&st_goal);
         break;
 
     case GAME_FALL:
         progress_stat(GAME_FALL);
-        gui_stuck();
-        goto_state(&st_fall_out);
+        goto_state(&st_fail);
         break;
 
     case GAME_TIME:
         progress_stat(GAME_TIME);
-        gui_stuck();
-        goto_state(&st_time_out);
+        goto_state(&st_fail);
         break;
 
     default:
@@ -372,39 +384,105 @@ static void play_loop_timer(int id, float dt)
     }
 }
 
-static void play_loop_point(int id, int x, int y, int dx, int dy)
+#ifdef __MAEMO__
+#define TOUCH_SENSITIVITY       20
+static int button_pressed = 0;
+static int button_pressed_x = 0;
+static int button_pressed_y = 0;
+static int dragging = 0;
+
+static void maemo_play_loop_point(int id, int x, int y, int dx, int dy)
 {
-    game_set_pos(dx, dy);
+    if (!dragging && button_pressed)
+    {
+        /* check how much we've moved */
+        if (abs(x - button_pressed_x) > TOUCH_SENSITIVITY ||
+            abs(y - button_pressed_y) > TOUCH_SENSITIVITY)
+            dragging = 1;
+    }
+
+    if (dragging)
+    {
+        if (dx > 0)
+        {
+            view_rotate.d = DIR_R;
+            view_rotate.R = dx / 15.0;
+        }
+        else
+        {
+            view_rotate.d = DIR_L;
+            view_rotate.L = dx / 15.0;
+        }
+    }
 }
 
-static void play_loop_stick(int id, int a, int k)
+static void maemo_play_loop_click(int b, int d)
+{
+    if (d)
+    {
+        int y;
+        SDL_GetMouseState(&button_pressed_x, &y);
+        button_pressed_y = -y + config_get_d(CONFIG_HEIGHT);
+    }
+    else
+    {
+        if (button_pressed && !dragging)
+            goto_state(&st_pause);
+        VIEWR_SET_R(0);
+        VIEWR_SET_L(0);
+    }
+    button_pressed = d;
+    dragging = 0;
+}
+#endif
+
+static void play_loop_point(int id, int x, int y, int dx, int dy)
+{
+#ifndef __MAEMO__
+    game_set_pos(dx, dy);
+#else
+    maemo_play_loop_point(id, x, y, dx, dy);
+#endif
+}
+
+static void play_loop_stick(int id, int a, float v, int bump)
 {
     if (config_tst_d(CONFIG_JOYSTICK_AXIS_X, a))
-        game_set_z(k);
+        game_set_z(v);
     if (config_tst_d(CONFIG_JOYSTICK_AXIS_Y, a))
-        game_set_x(k);
+        game_set_x(v);
     if (config_tst_d(CONFIG_JOYSTICK_AXIS_U, a))
-        view_rotate = (float) k / 32768.0f;
+    {
+        VIEWR_SET_R(0);
+        VIEWR_SET_L(0);
+
+        if (v > 0) VIEWR_SET_R(v);
+        if (v < 0) VIEWR_SET_L(v);
+    }
 }
 
 static int play_loop_click(int b, int d)
 {
+#ifndef __MAEMO__
     if (d)
     {
         if (config_tst_d(CONFIG_MOUSE_CAMERA_R, b))
-            view_rotate = +1;
+            VIEWR_SET_R(+1);
         if (config_tst_d(CONFIG_MOUSE_CAMERA_L, b))
-            view_rotate = -1;
+            VIEWR_SET_L(-1);
 
         click_camera(b);
     }
     else
     {
         if (config_tst_d(CONFIG_MOUSE_CAMERA_R, b))
-            view_rotate = 0;
+            VIEWR_SET_R(0);
         if (config_tst_d(CONFIG_MOUSE_CAMERA_L, b))
-            view_rotate = 0;
+            VIEWR_SET_L(0);
     }
+#else
+    maemo_play_loop_click(b, d);
+#endif
 
     return 1;
 }
@@ -414,9 +492,9 @@ static int play_loop_keybd(int c, int d)
     if (d)
     {
         if (config_tst_d(CONFIG_KEY_CAMERA_R, c))
-            view_rotate = +1;
+            VIEWR_SET_R(+1);
         if (config_tst_d(CONFIG_KEY_CAMERA_L, c))
-            view_rotate = -1;
+            VIEWR_SET_L(-1);
         if (config_tst_d(CONFIG_KEY_ROTATE_FAST, c))
             fast_rotate = 1;
 
@@ -429,14 +507,14 @@ static int play_loop_keybd(int c, int d)
                 goto_state(&st_play_ready);
         }
         if (config_tst_d(CONFIG_KEY_PAUSE, c))
-            goto_pause();
+            goto_state(&st_pause);
     }
     else
     {
         if (config_tst_d(CONFIG_KEY_CAMERA_R, c))
-            view_rotate = 0;
+            VIEWR_SET_R(0);
         if (config_tst_d(CONFIG_KEY_CAMERA_L, c))
-            view_rotate = 0;
+            VIEWR_SET_L(0);
         if (config_tst_d(CONFIG_KEY_ROTATE_FAST, c))
             fast_rotate = 0;
     }
@@ -463,9 +541,9 @@ static int play_loop_buttn(int b, int d)
             pause_or_exit();
 
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_R, b))
-            view_rotate = +1;
+            VIEWR_SET_R(+1);
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L, b))
-            view_rotate = -1;
+            VIEWR_SET_L(-1);
         if (config_tst_d(CONFIG_JOYSTICK_ROTATE_FAST, b))
             fast_rotate = 1;
 
@@ -474,9 +552,9 @@ static int play_loop_buttn(int b, int d)
     else
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_R, b))
-            view_rotate = 0;
+            VIEWR_SET_R(0);
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L, b))
-            view_rotate = 0;
+            VIEWR_SET_L(0);
         if (config_tst_d(CONFIG_JOYSTICK_ROTATE_FAST, b))
             fast_rotate = 0;
     }
@@ -488,20 +566,20 @@ static int play_loop_buttn(int b, int d)
 static float phi;
 static float theta;
 
-static int look_enter(void)
+static int look_enter(struct state *st, struct state *prev)
 {
     phi   = 0;
     theta = 0;
     return 0;
 }
 
-static void look_leave(int id)
+static void look_leave(struct state *st, struct state *next, int id)
 {
 }
 
 static void look_paint(int id, float t)
 {
-    game_draw(0, t);
+    game_client_draw(0, t);
 }
 
 static void look_point(int id, int x, int y, int dx, int dy)
@@ -546,8 +624,7 @@ struct state st_play_ready = {
     NULL,
     play_ready_click,
     play_ready_keybd,
-    play_ready_buttn,
-    1, 0
+    play_ready_buttn
 };
 
 struct state st_play_set = {
@@ -560,8 +637,7 @@ struct state st_play_set = {
     NULL,
     play_set_click,
     play_set_keybd,
-    play_set_buttn,
-    1, 0
+    play_set_buttn
 };
 
 struct state st_play_loop = {
@@ -574,8 +650,7 @@ struct state st_play_loop = {
     shared_angle,
     play_loop_click,
     play_loop_keybd,
-    play_loop_buttn,
-    0, 0
+    play_loop_buttn
 };
 
 struct state st_look = {
@@ -588,6 +663,5 @@ struct state st_look = {
     NULL,
     NULL,
     look_keybd,
-    look_buttn,
-    0, 0
+    look_buttn
 };
