@@ -25,7 +25,7 @@ int  curr_count(void);
 const char *curr_scr(void);
 const char *curr_par(void);
 
-void hole_goto(int, int);
+int  hole_goto(int, int);
 int  hole_next(void);
 int  hole_move(void);
 void hole_goal(void);
