@@ -32,7 +32,6 @@ int CONFIG_HEIGHT;
 int CONFIG_STEREO;
 int CONFIG_CAMERA;
 int CONFIG_TEXTURES;
-int CONFIG_GEOMETRY;
 int CONFIG_REFLECTION;
 int CONFIG_MULTISAMPLE;
 int CONFIG_MIPMAP;
@@ -58,6 +57,9 @@ int CONFIG_JOYSTICK_DEVICE;
 int CONFIG_JOYSTICK_AXIS_X;
 int CONFIG_JOYSTICK_AXIS_Y;
 int CONFIG_JOYSTICK_AXIS_U;
+int CONFIG_JOYSTICK_AXIS_X_INVERT;
+int CONFIG_JOYSTICK_AXIS_Y_INVERT;
+int CONFIG_JOYSTICK_AXIS_U_INVERT;
 int CONFIG_JOYSTICK_BUTTON_A;
 int CONFIG_JOYSTICK_BUTTON_B;
 int CONFIG_JOYSTICK_BUTTON_R;
@@ -78,12 +80,6 @@ int CONFIG_KEY_CAMERA_3;
 int CONFIG_KEY_CAMERA_TOGGLE;
 int CONFIG_KEY_CAMERA_R;
 int CONFIG_KEY_CAMERA_L;
-int CONFIG_VIEW_FOV;
-int CONFIG_VIEW_DP;
-int CONFIG_VIEW_DC;
-int CONFIG_VIEW_DZ;
-int CONFIG_ROTATE_FAST;
-int CONFIG_ROTATE_SLOW;
 int CONFIG_KEY_FORWARD;
 int CONFIG_KEY_BACKWARD;
 int CONFIG_KEY_LEFT;
@@ -92,9 +88,14 @@ int CONFIG_KEY_PAUSE;
 int CONFIG_KEY_RESTART;
 int CONFIG_KEY_SCORE_NEXT;
 int CONFIG_KEY_ROTATE_FAST;
+int CONFIG_VIEW_FOV;
+int CONFIG_VIEW_DP;
+int CONFIG_VIEW_DC;
+int CONFIG_VIEW_DZ;
+int CONFIG_ROTATE_FAST;
+int CONFIG_ROTATE_SLOW;
 int CONFIG_CHEAT;
 int CONFIG_STATS;
-int CONFIG_UNIFORM;
 int CONFIG_SCREENSHOT;
 int CONFIG_LOCK_GOALS;
 
@@ -114,13 +115,12 @@ static struct
     const int   def;
     int         cur;
 } option_d[] = {
-    { &CONFIG_FULLSCREEN,   "fullscreen",   0 },
+    { &CONFIG_FULLSCREEN,   "fullscreen",   1 },
     { &CONFIG_WIDTH,        "width",        800 },
-    { &CONFIG_HEIGHT,       "height",       600 },
+    { &CONFIG_HEIGHT,       "height",       480 },
     { &CONFIG_STEREO,       "stereo",       0 },
     { &CONFIG_CAMERA,       "camera",       0 },
     { &CONFIG_TEXTURES,     "textures",     1 },
-    { &CONFIG_GEOMETRY,     "geometry",     1 },
     { &CONFIG_REFLECTION,   "reflection",   1 },
     { &CONFIG_MULTISAMPLE,  "multisample",  0 },
     { &CONFIG_MIPMAP,       "mipmap",       0 },
@@ -149,6 +149,9 @@ static struct
     { &CONFIG_JOYSTICK_AXIS_X,        "joystick_axis_x",        0 },
     { &CONFIG_JOYSTICK_AXIS_Y,        "joystick_axis_y",        1 },
     { &CONFIG_JOYSTICK_AXIS_U,        "joystick_axis_u",        2 },
+    { &CONFIG_JOYSTICK_AXIS_X_INVERT, "joystick_axis_x_invert", 0 },
+    { &CONFIG_JOYSTICK_AXIS_Y_INVERT, "joystick_axis_y_invert", 0 },
+    { &CONFIG_JOYSTICK_AXIS_U_INVERT, "joystick_axis_u_invert", 0 },
     { &CONFIG_JOYSTICK_BUTTON_A,      "joystick_button_a",      0 },
     { &CONFIG_JOYSTICK_BUTTON_B,      "joystick_button_b",      1 },
     { &CONFIG_JOYSTICK_BUTTON_R,      "joystick_button_r",      2 },
@@ -170,12 +173,6 @@ static struct
     { &CONFIG_KEY_CAMERA_TOGGLE, "key_camera_toggle", SDLK_e },
     { &CONFIG_KEY_CAMERA_R,      "key_camera_r",      SDLK_d },
     { &CONFIG_KEY_CAMERA_L,      "key_camera_l",      SDLK_s },
-    { &CONFIG_VIEW_FOV,          "view_fov",          50 },
-    { &CONFIG_VIEW_DP,           "view_dp",           75 },
-    { &CONFIG_VIEW_DC,           "view_dc",           25 },
-    { &CONFIG_VIEW_DZ,           "view_dz",           200 },
-    { &CONFIG_ROTATE_FAST,       "rotate_fast",       300 },
-    { &CONFIG_ROTATE_SLOW,       "rotate_slow",       150 },
     { &CONFIG_KEY_FORWARD,       "key_forward",       SDLK_UP },
     { &CONFIG_KEY_BACKWARD,      "key_backward",      SDLK_DOWN },
     { &CONFIG_KEY_LEFT,          "key_left",          SDLK_LEFT },
@@ -184,11 +181,17 @@ static struct
     { &CONFIG_KEY_RESTART,       "key_restart",       SDLK_r },
     { &CONFIG_KEY_SCORE_NEXT,    "key_score_next",    SDLK_TAB },
     { &CONFIG_KEY_ROTATE_FAST,   "key_rotate_fast",   SDLK_LSHIFT },
-    { &CONFIG_CHEAT,             "cheat",             0 },
-    { &CONFIG_STATS,             "stats",             0 },
-    { &CONFIG_UNIFORM,           "uniform",           0 },
-    { &CONFIG_SCREENSHOT,        "screenshot",        0 },
-    { &CONFIG_LOCK_GOALS,        "lock_goals",        0 }
+
+    { &CONFIG_VIEW_FOV,    "view_fov",    50 },
+    { &CONFIG_VIEW_DP,     "view_dp",     75 },
+    { &CONFIG_VIEW_DC,     "view_dc",     25 },
+    { &CONFIG_VIEW_DZ,     "view_dz",     200 },
+    { &CONFIG_ROTATE_FAST, "rotate_fast", 300 },
+    { &CONFIG_ROTATE_SLOW, "rotate_slow", 150 },
+    { &CONFIG_CHEAT,       "cheat",       0 },
+    { &CONFIG_STATS,       "stats",       0 },
+    { &CONFIG_SCREENSHOT,  "screenshot",  0 },
+    { &CONFIG_LOCK_GOALS,  "lock_goals",  0 }
 };
 
 static struct
@@ -285,55 +288,33 @@ void config_init(void)
 }
 
 /*
- * Scan a NUL-terminated string LINE according to the format
- * '^<space>?<key><space><value>$' and store pointers to the start of key and
- * value at DST_KEY and DST_VAL, respectively.  No memory is allocated to store
- * the strings;  instead, the memory pointed to by LINE modified in-place as
- * needed.
- *
- * Return 1 if LINE matches the format, return 0 otherwise.
+ * Scan an option string and store pointers to the start of key and
+ * value at the passed-in locations.  No memory is allocated to store
+ * the strings; instead, the option string is modified in-place as
+ * needed.  Return 1 on success, 0 on error.
  */
-
 static int scan_key_and_value(char **dst_key, char **dst_val, char *line)
 {
     if (line)
     {
-        char *key, *val, *space;
+        int ks, ke, vs;
 
-        for (key = line; *key && isspace(*key); key++);
+        ks = -1;
+        ke = -1;
+        vs = -1;
 
-        if (*key)
-        {
-            if (dst_key)
-                *dst_key = key;
-        }
-        else
+        sscanf(line, " %n%*s%n %n", &ks, &ke, &vs);
+
+        if (ks < 0 || ke < 0 || vs < 0)
             return 0;
 
-        for (space = key; *space && !isspace(*space); space++);
-
-        if (*space)
-        {
-            /* NUL-terminate the key, if necessary. */
-
-            if (dst_key)
-            {
-                *space = '\0';
-                space++;
-            }
-        }
-        else
+        if (vs - ke < 1)
             return 0;
 
-        for (val = space; *val && isspace(*val); val++);
+        line[ke] = 0;
 
-        if (*val)
-        {
-            if (dst_val)
-                *dst_val = val;
-        }
-        else
-            return 0;
+        *dst_key = line + ks;
+        *dst_val = line + vs;
 
         return 1;
     }
