@@ -28,6 +28,13 @@
 #include "base_config.h"
 #include "lang.h"
 
+#ifdef __MAEMO__
+#include <SDL_gles.h>
+#define SDL_GL_SetAttribute(attr, value)  SDL_GLES_SetAttribute(attr, value)
+#define SDL_GL_GetAttribute(attr, value)  SDL_GLES_GetAttribute(attr, value)
+#define SDL_GL_SwapBuffers()              SDL_GLES_SwapBuffers()
+#endif
+
 /*---------------------------------------------------------------------------*/
 
 /* Integer options. */
@@ -38,7 +45,6 @@ extern int CONFIG_HEIGHT;
 extern int CONFIG_STEREO;
 extern int CONFIG_CAMERA;
 extern int CONFIG_TEXTURES;
-extern int CONFIG_GEOMETRY;
 extern int CONFIG_REFLECTION;
 extern int CONFIG_MULTISAMPLE;
 extern int CONFIG_MIPMAP;
@@ -64,6 +70,9 @@ extern int CONFIG_JOYSTICK_DEVICE;
 extern int CONFIG_JOYSTICK_AXIS_X;
 extern int CONFIG_JOYSTICK_AXIS_Y;
 extern int CONFIG_JOYSTICK_AXIS_U;
+extern int CONFIG_JOYSTICK_AXIS_X_INVERT;
+extern int CONFIG_JOYSTICK_AXIS_Y_INVERT;
+extern int CONFIG_JOYSTICK_AXIS_U_INVERT;
 extern int CONFIG_JOYSTICK_BUTTON_A;
 extern int CONFIG_JOYSTICK_BUTTON_B;
 extern int CONFIG_JOYSTICK_BUTTON_R;
@@ -84,12 +93,6 @@ extern int CONFIG_KEY_CAMERA_3;
 extern int CONFIG_KEY_CAMERA_TOGGLE;
 extern int CONFIG_KEY_CAMERA_R;
 extern int CONFIG_KEY_CAMERA_L;
-extern int CONFIG_VIEW_FOV;
-extern int CONFIG_VIEW_DP;
-extern int CONFIG_VIEW_DC;
-extern int CONFIG_VIEW_DZ;
-extern int CONFIG_ROTATE_FAST;
-extern int CONFIG_ROTATE_SLOW;
 extern int CONFIG_KEY_FORWARD;
 extern int CONFIG_KEY_BACKWARD;
 extern int CONFIG_KEY_LEFT;
@@ -98,9 +101,14 @@ extern int CONFIG_KEY_PAUSE;
 extern int CONFIG_KEY_RESTART;
 extern int CONFIG_KEY_SCORE_NEXT;
 extern int CONFIG_KEY_ROTATE_FAST;
+extern int CONFIG_VIEW_FOV;
+extern int CONFIG_VIEW_DP;
+extern int CONFIG_VIEW_DC;
+extern int CONFIG_VIEW_DZ;
+extern int CONFIG_ROTATE_FAST;
+extern int CONFIG_ROTATE_SLOW;
 extern int CONFIG_CHEAT;
 extern int CONFIG_STATS;
-extern int CONFIG_UNIFORM;
 extern int CONFIG_SCREENSHOT;
 extern int CONFIG_LOCK_GOALS;
 
