@@ -22,6 +22,7 @@
 #include "part.h"
 #include "vec3.h"
 #include "image.h"
+#include "geom.h"
 
 /*---------------------------------------------------------------------------*/
 /*
@@ -236,11 +237,15 @@ void part_draw_coin(struct s_rend *rend)
     glBindBuffer_(GL_ARRAY_BUFFER, 0);
 #endif
 
-    glClientActiveTexture_(GL_TEXTURE2);
-    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-    glClientActiveTexture_(GL_TEXTURE1);
-    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-    glClientActiveTexture_(GL_TEXTURE0);
+    if (tex_env_stage(TEX_STAGE_SHADOW))
+    {
+        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+        if (tex_env_stage(TEX_STAGE_CLIP))
+            glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+        tex_env_stage(TEX_STAGE_TEXTURE);
+    }
     glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 
     glDisableClientState(GL_NORMAL_ARRAY);
@@ -269,12 +274,16 @@ void part_draw_coin(struct s_rend *rend)
     glDisableClientState(GL_COLOR_ARRAY);
     glEnableClientState(GL_NORMAL_ARRAY);
 
+    if (tex_env_stage(TEX_STAGE_SHADOW))
+    {
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+
+        if (tex_env_stage(TEX_STAGE_CLIP))
+            glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+
+        tex_env_stage(TEX_STAGE_TEXTURE);
+    }
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-    glClientActiveTexture_(GL_TEXTURE2);
-    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-    glClientActiveTexture_(GL_TEXTURE1);
-    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-    glClientActiveTexture_(GL_TEXTURE0);
 }
 
 /*---------------------------------------------------------------------------*/
